<?php

/**
 * Settings class; Responsible for admin options page
 */
class ESS_Settings
{
    private $settings,
			$buttons = array(
				'facebook' => 'Facebook',
				'twitter' => 'Twitter',
				'googleplus' => 'Google+',
				'pinterest' => 'Pinterest',
				'linkedin' => 'LinkedIn',
				'whatsapp' => 'Whatsapp'
			),
			$buttons_placements = array(
				'before' => 'Before post content',
				'after' => 'After post content',
				'float' => 'Floating on the left',
				'featured_image' => 'Inside featured image',
			),
			$buttons_sizes = array(
				'small' => 'Small',
				'medium' => 'Medium',
				'large' => 'Large',
			),
			$options;

	public function __construct( $file )
	{
		$this->options = new ESS_Options( ESS_SLUG . '_' );

		// Initialise settings
		add_action( 'admin_init', array( $this, 'essinit' ) );

		// Register plugin settings
		add_action( 'admin_init' , array( $this, 'ess_register_settings' ) );

		// Add settings page to menu
		add_action( 'admin_menu' , array( $this, 'add_menu_item' ) );

		// Add settings link to plugins page
		add_filter( 'plugin_action_links_' . plugin_basename( $file ) , array( $this, 'ess_add_settings_link' ) );
	}

	/**
	 * Initialise settings
	 *
	 * @return void
	 */
	public function essinit()
	{
		$this->settings = $this->ess_settings_fields();
	}

	/**
	 * Add settings page to admin menu
	 * @return void
	 */
	public function add_menu_item()
	{
		$page = add_options_page( __( ESS_NAME . ' Settings', ESS_TEXT_DOMAIN ) , __( ESS_NAME, ESS_TEXT_DOMAIN ) , 'manage_options' , ESS_SLUG . '-settings' ,  array( $this, 'settings_page' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'ess_settings_assets' ) );
	}

	/**
	 * Load settings JS & CSS
	 * @return void
	 */
	public function ess_settings_assets( $hook )
	{
		// don't enqueue assets on pages other than ESS Settings
		if( $hook != 'settings_page_' . ESS_SLUG . '-settings' ) {
        	return;
        }

		// Farbtastic color picker bundled with WP
		wp_enqueue_style( 'farbtastic' );
    	wp_enqueue_script( 'farbtastic' );

    	// Custom styling for settings page
    	wp_enqueue_style( ESS_SLUG . '-settings', ESS_ASSETS_URL . 'css/settings.css' );

    	// jQuery UI Sortable for buttons order; drag n drop
    	wp_enqueue_script( 'jquery-ui-sortable' );

	    wp_enqueue_media();

	    wp_register_script( ESS_SLUG . '-settings', ESS_ASSETS_URL . 'js/settings.js', array( 'farbtastic', 'jquery', 'jquery-ui-sortable' ), ESS_VERSION );
	    wp_enqueue_script( ESS_SLUG . '-settings' );
	}

	/**
	 * Add Settings option on plugins list page linking to settins page
	 *
	 * @param  array $links Existing links
	 * @return array 		Modified links
	 */
	public function ess_add_settings_link( $links ) {
		$settings_link = '<a href="options-general.php?page=' . ESS_SLUG . '-settings">' . __( 'Settings', ESS_TEXT_DOMAIN ) . '</a>';
  		array_unshift( $links, $settings_link );

  		return $links;
	}

	/**
	 * Construct settings fields
	 *
	 * @return array Fields to be rendered on settings page
	 */
	private function ess_settings_fields()
	{
		$settings['settings'] = array(
			'title'					=> __( 'Settings', ESS_TEXT_DOMAIN ),
			'description'			=> __( 'Adjust these settings to get the most out of Easy Social Sharebar Settings', ESS_TEXT_DOMAIN ),
			'fields'				=> array(
				array(
					'id' 			=> 'post_types',
					'label'			=> __( 'Post Types', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Sharebar will be displayed on these post types automatically.', ESS_TEXT_DOMAIN ),
					'type'			=> 'checkbox_multi',
					'options'		=> $this->ess_get_post_types(),
					'default'		=> array()
				),
				array(
					'id' 			=> 'buttons',
					'label'			=> __( 'Social Buttons', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Select which sharing buttons you would like to display.', ESS_TEXT_DOMAIN ),
					'type'			=> 'checkbox_multi',
					'options'		=> $this->buttons,
					'default'		=> array()
				),
				array(
					'id' 			=> 'buttons_order',
					'label'			=> __( 'Buttons Order', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Drag \'n Drop to change button order.' , ESS_TEXT_DOMAIN ),
					'type'			=> 'draggable',
					'options'		=> array(),
					'default'		=> array()
				),
				array(
					'id' 			=> 'twitter_username',
					'label'			=> __( 'Twitter Username', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Set this if you want to append "via @username" to tweets. Don\'t include @ in the username.', ESS_TEXT_DOMAIN ),
					'type'			=> 'text',
					'default'		=> '',
					'placeholder'	=> __( 'testuser', ESS_TEXT_DOMAIN )
				),
				array(
					'id' 			=> 'fallback_image',
					'label'			=> __( 'Default image for Pinterest' , ESS_TEXT_DOMAIN ),
					'description'	=> __( 'This will be used as fallback for Pinterest pin if a post has no thumbnail.', ESS_TEXT_DOMAIN ),
					'type'			=> 'media',
					'default'		=> '',
					'placeholder'	=> ''
				),
				array(
					'id' 			=> 'buttons_placement',
					'label'			=> __( 'Buttons Placement', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Select where you would like the sharing buttons to display. You can choose more than one.<br>Floating sharebar will disapear on smaller screens.', ESS_TEXT_DOMAIN ),
					'type'			=> 'checkbox_multi',
					'options'		=> $this->buttons_placements,
					'default'		=> array()
				),
				array(
					'id' 			=> 'buttons_size',
					'label'			=> __( 'Buttons Size', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Choose whichever size suits your site.' , ESS_TEXT_DOMAIN ),
					'type'			=> 'radio',
					'options'		=> $this->buttons_sizes,
					'default'		=> array()
				),
				array(
					'id' 			=> 'buttons_color',
					'label'			=> __( 'Custom buttons color', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Fancy a custom color for the share buttons? Click in the box above and choose', ESS_TEXT_DOMAIN ),
					'type'			=> 'color',
					'default'		=> '#'
				),
				array(
					'id' 			=> 'buttons_hover_color',
					'label'			=> __( 'Buttons hover-state color', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Click in the box above to choose a custom hover background color for the buttons', ESS_TEXT_DOMAIN ),
					'type'			=> 'color',
					'default'		=> '#'
				),
				array(
					'id' 			=> 'exclude_fontawesome',
					'label'			=> __( 'Exclude Font Awesome?', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Check this if you want the plugin not to load Font Awesome.<br>It can be helpful if Font Awesome is already loaded by another plugin or theme.', ESS_TEXT_DOMAIN ),
					'type'			=> 'checkbox',
					'default'		=> ''
				),
				array(
					'id' 			=> 'include_social_meta',
					'label'			=> __( 'Include social meta tags?', ESS_TEXT_DOMAIN ),
					'description'	=> __( 'Having social meta tags included will enhance your posts visibilty on social networks.<br>This is mandatory if you\'re not using any SEO plugin.' , ESS_TEXT_DOMAIN ),
					'type'			=> 'checkbox',
					'default'		=> ''
				),
			)
		);

		// $settings = apply_filters( ESS_SETTINGS_SLUG . '_fields', $settings );

		return $settings;
	}

	/**
	 * Register settings
	 *
	 * @return void
	 */
	public function ess_register_settings()
	{
		if( is_array( $this->settings ) ) {
			foreach( $this->settings as $section => $data ) {

				add_settings_section(
					$section,
					$data['title'],
					array( $this, 'settings_section' ),
					ESS_SETTINGS_SLUG
				);

				foreach( $data['fields'] as $field ) {
					$validation = '';

					if( isset( $field['callback'] ) ) {
						$validation = $field['callback'];
					}

					$option_name = $this->options->get_option_name( $field['id'] );
					register_setting(
						ESS_SETTINGS_SLUG,
						$option_name,
						$validation
					);

					add_settings_field(
						$field['id'],
						$field['label'],
						array( $this, 'render_field' ),
						ESS_SETTINGS_SLUG,
						$section,
						array( 'field' => $field )
					);
				}
			}
		}
	}

	public function settings_section( $section )
	{
		echo '<p id="' . $section['id'] . '_description">' . $this->settings[ $section['id'] ]['description'] . '</p>' . "\n";
	}

	/**
	 * Render field to display on settings page
	 *
	 * @param  array $args Field data
	 * @return void
	 */
	public function render_field( $args )
	{
		$field = $args['field'];

		$html = '';

		$option_name = $this->options->get_option_name( $field['id'] );
		$option = $this->options->get( $field['id'], '' );

		$data = '';
		if ( isset( $field['default'] ) ) {
			$data = $field['default'];
		}

		if ( $option ) {
			$data = $option;
		}

		switch( $field['type'] ) {


			case 'checkbox_multi':
				$html .= '<ul id="' . $field['id'] . '">';

				foreach( $field['options'] as $k => $v ) {
					$checked = false;

					if( in_array( $k, $data ) ) {
						$checked = true;
					}

					$html .= '<li><label for="' . esc_attr($field['id'] . '_' . $k) . '"><input type="checkbox" ' . checked( $checked, true, false ) . ' name="' . esc_attr($option_name) . '[]" value="' . esc_attr( $k ) . '" id="' . esc_attr( $field['id'] . '_' . $k ) . '" /> ' . $v . '</label></li>';
				}

				$html .= '<ul>';
			break;

			case 'draggable':
				$html .= '<div id="buttons-order">';

				if ( !is_array( $data ) ) {
					$data = explode( ',', $data );
				}

				$buttons = $this->options->get( 'buttons', array() );

				foreach( $data as $button ) {
					$html .= '<div id="' . $button . '" class="draggable-element"';

					if ( !in_array( $button, $buttons ) ) {
						$html .= ' style="display: none;"';
					}

					$html .= '><label>' . $this->buttons[$button] . '</label>';
					$html .= '<div class="draggable-indicator"><svg viewBox="0 0 32 32"><rect height="4" width="4" y="4" x="0"></rect><rect height="4" width="4" y="12" x="0"></rect><rect height="4" width="4" y="20" x="0"></rect><rect height="4" width="4" y="28" x="0"></rect><rect height="4" width="4" y="4" x="8"></rect><rect height="4" width="4" y="12" x="8"></rect><rect height="4" width="4" y="20" x="8"></rect><rect height="4" width="4" y="28" x="8"></rect><rect height="4" width="4" y="4" x="16"></rect><rect height="4" width="4" y="12" x="16"></rect><rect height="4" width="4" y="20" x="16"></rect><rect height="4" width="4" y="28" x="16"></rect><rect height="4" width="4" y="4" x="24"></rect><rect height="4" width="4" y="12" x="24"></rect><rect height="4" width="4" y="20" x="24"></rect><rect height="4" width="4" y="28" x="24"></rect></svg></div>';
					$html .=' </div>';
				}

				$html .= '<div>';
				$html .= '<input id="' . esc_attr( $field['id'] ) . '" type="hidden" name="' . esc_attr( $option_name ) . '" value="' . implode( ',', $data ) . '">';
			break;

			case 'text':
				$html .= '<input id="' . esc_attr( $field['id'] ) . '" type="' . $field['type'] . '" name="' . esc_attr( $option_name ) . '" placeholder="' . esc_attr( $field['placeholder'] ) . '" value="' . $data . '"/><br>';
			break;

			case 'media':
				$image = '';
				if( $data ) {
					$image = wp_get_attachment_thumb_url( $data );
				}
				$html .= '<img id="' . $option_name . '_preview" class="image_preview" src="' . $image . '" /><br/>' . "\n";
				$html .= '<input id="' . $option_name . '_button" type="button" data-uploader_title="' . __( 'Upload an image' , 'plugin_textdomain' ) . '" data-uploader_button_text="' . __( 'Use image' , ESS_TEXT_DOMAIN ) . '" class="image_upload_button button" value="'. __( 'Upload new image' , ESS_TEXT_DOMAIN ) . '" />' . "\n";
				$html .= '<input id="' . $option_name . '_delete" type="button" class="image_delete_button button" value="'. __( 'Remove image' , 'plugin_textdomain' ) . '" />' . "\n";
				$html .= '<input id="' . $option_name . '" class="image_data_field" type="hidden" name="' . $option_name . '" value="' . $data . '"/><br/>' . "\n";
			break;

			case 'radio':
				$html .= '<ul id="' . $field['id'] .'">';

				foreach($field['options'] as $k => $v) {
					$checked = false;

					if($k == $data) {
						$checked = true;
					}

					$html .= '<li>';
					$html .= '<label for="' . esc_attr($field['id'] . '_' . $k) . '"><input type="radio" ' . checked($checked, true, false) . ' name="' . esc_attr($option_name) . '" value="' . esc_attr($k) . '" id="' . esc_attr($field['id'] . '_' . $k) . '" /> ' . $v . '</label>';


					if ( $field[ 'id' ] == 'buttons_size' ) {
						$html .= '<img class="helper-image" src="' . ESS_ASSETS_URL . 'images/ess-button-' . $k . '.png">';
					}

					$html .= '</li>';
				}

				$html .= '</ul>';
			break;

			case 'checkbox':
				$checked = '';
				if( $option && 'on' == $option ){
					$checked = 'checked="checked"';
				}
				$html .= '<input id="' . esc_attr( $field['id'] ) . '" type="' . $field['type'] . '" name="' . esc_attr( $option_name ) . '" ' . $checked . '/>' . "\n";
			break;

			case 'color': ?>
				<div class="color-picker" style="position:relative;">
			        <input type="text" name="<?php esc_attr_e( $option_name ); ?>" class="color" value="<?php esc_attr_e( $data ); ?>" />
			        <a href="#" class="ess-use-default dashicons-before dashicons-dismiss" title="Use default colors">Use default colors</a>
			        <div style="position:absolute;background:#FFF;z-index:99;border-radius:100%;" class="color-picker-input"  data-section="settings"></div>
			    </div>
			    <?php
			break;

		}

		switch( $field['type'] ) {

			case 'checkbox_multi':
			case 'radio':
				$html .= '<span class="description">' . $field['description'] . '</span>';
				break;

			default:
				$html .= '<label for="' . esc_attr( $field['id'] ) . '"><span class="description">' . $field['description'] . '</span></label>' . "\n";
			break;
		}

		echo $html;
	}

	/**
	 * Validate individual settings field
	 *
	 * @param  string $data Inputted value
	 * @return string       Validated value
	 */
	public function validate_field( $data )
	{
		if( $data && strlen( $data ) > 0 && $data != '' ) {
			$data = urlencode( strtolower( str_replace( ' ' , '-' , $data ) ) );
		}

		return $data;
	}

	/**
	 * Render settings page content
	 *
	 * @return void
	 */
	public function settings_page()
	{
		$html = '<div class="wrap" id="' . ESS_SETTINGS_SLUG . '">' . "\n";
			$html .= '<h2>' . __( ESS_NAME , ESS_TEXT_DOMAIN ) . '</h2>' . "\n";
			$html .= '<form method="post" action="options.php" enctype="multipart/form-data">' . "\n";

				// Render settings fields
				ob_start();
				settings_fields( ESS_SETTINGS_SLUG );
				do_settings_sections( ESS_SETTINGS_SLUG );
				$html .= ob_get_clean();

				$html .= '<p class="submit">' . "\n";
					$html .= '<input name="Submit" type="submit" class="button-primary" value="' . esc_attr( __( 'Save Settings' , ESS_TEXT_DOMAIN ) ) . '" />' . "\n";
				$html .= '</p>' . "\n";
			$html .= '</form>' . "\n";

			$html .= $this->ess_render_shortcode_generator_section();

		$html .= '</div>' . "\n";

		echo $html;
	}

	/**
	 * Prepares HTML for the Shortcode Genrerator section
	 * @return string Shortcode Generator section HTML
	 */
	private function ess_render_shortcode_generator_section()
	{
		ob_start();
		?>
			<div id="generate-shortcode" class="wrap narrow">
				<hr>
		    	<h2 class="title">Generate Shortcode</h2>
		    	<p>Customize the following attributes to get a shortcode tailered for your needs.</p>
		        <table class="form-table">
		            <tbody>
		                <tr>
		                    <th valign="top" scope="row">
		                        <label for="shortcode-buttons">Social Buttons</label>
		                    </th>
		                    <td>
								<ul id="shortcode-buttons">
		                    	<?php foreach( $this->buttons as $slug => $name ) : ?>
									<li>
										<label for="shortcode-buttons-<?php echo $slug; ?>">
											<input type="checkbox" id="shortcode-buttons-<?php echo $slug; ?>" value="<?php echo $slug; ?>" <?php checked( true, true, true ); ?>/><?php echo $name; ?>
										</label>
									</li>
								<?php endforeach; ?>
								<ul>
		                    </td>
		                </tr>

		                <tr>
		                	<th valign="top" scope="row">
		                		<label for="shortcode-buttons-order">Buttons Order</label>
		                	</th>
		                	<td>
		                		<div id="shortcode-buttons-order">
								<?php foreach( $this->buttons as $slug => $name ) : ?>
									<div id="shortcode-draggable-<?php echo $slug; ?>" class="draggable-element" data-button="<?php echo $slug; ?>">
										<label><?php echo $name; ?></label>
										<div class="draggable-indicator"><svg viewBox="0 0 32 32"><rect height="4" width="4" y="4" x="0"></rect><rect height="4" width="4" y="12" x="0"></rect><rect height="4" width="4" y="20" x="0"></rect><rect height="4" width="4" y="28" x="0"></rect><rect height="4" width="4" y="4" x="8"></rect><rect height="4" width="4" y="12" x="8"></rect><rect height="4" width="4" y="20" x="8"></rect><rect height="4" width="4" y="28" x="8"></rect><rect height="4" width="4" y="4" x="16"></rect><rect height="4" width="4" y="12" x="16"></rect><rect height="4" width="4" y="20" x="16"></rect><rect height="4" width="4" y="28" x="16"></rect><rect height="4" width="4" y="4" x="24"></rect><rect height="4" width="4" y="12" x="24"></rect><rect height="4" width="4" y="20" x="24"></rect><rect height="4" width="4" y="28" x="24"></rect></svg></div>
									</div>
								<?php endforeach; ?>
								</div>
		                	</td>
		                </tr>

		                <tr>
		                	<th valign="top" scope="row">
		                		<label for="shortcode-buttons-size">Buttons Size</label>
		                	</th>
		                	<td>
		                		<ul id="shortcode-buttons-size">
								<?php foreach( $this->buttons_sizes as $size => $name ) : ?>
									<li>
		                				<label for="shortcode-size-<?php echo $size; ?>">
		                					<input type="radio" id="shortcode-size-<?php echo $size; ?>" name="shortcode_buttons_size" value="<?php echo $size; ?>" <?php checked( $size == 'medium', true, true ); ?>> <?php echo $name; ?>
		                				</label>
		                			</li>
								<?php endforeach; ?>
								</ul>
		                	</td>
		                </tr>

		                <tr>
		                	<th valign="top" scope="row">
		                		<label for="shortcode-buttons-color">Custom buttons color</label>
		                	</th>
		                	<td>
		                		<div class="color-picker" style="position:relative;">
							        <input type="text" id="shortcode-buttons-color-value" class="color" value="#" data-type="color"/>
							        <a href="#" class="ess-use-default ess-shortcode-use-default dashicons-before dashicons-dismiss" title="Use default colors">Use default colors</a>
							        <div style="position:absolute;background:#FFF;z-index:99;border-radius:100%;" class="color-picker-input" data-section="shortcode"></div>
							    </div>
		                	</td>
		                </tr>

		                <tr>
		                	<th valign="top" scope="row">
		                		<label for="shortcode-buttons-hover-color">Buttons hover-state color</label>
		                	</th>
		                	<td>
		                		<div class="color-picker" style="position:relative;">
							        <input type="text" id="shortcode-buttons-hover-color-value" class="color" value="#" data-type="hover_color"/>
							        <a href="#" class="ess-use-default ess-shortcode-use-default dashicons-before dashicons-dismiss" title="Use default colors">Use default colors</a>
							        <div style="position:absolute;background:#FFF;z-index:99;border-radius:100%;" class="color-picker-input" data-section="shortcode"></div>
							    </div>
		                	</td>
		                </tr>

		                <tr>
		                	<th>
		        		        <p class="submit">
						            <input id="reset-shortcode-inputs" type="submit" class="button" value="Reset">
						        </p>
		                	</th>
		                	<td>
		                		<span id="ess-shortcode-update-notice" class="description">Updated...</span>
		                		<input type="text" id="shortcode-holder" onfocus="this.select();" readonly="readonly" value="[ess]" class="large-text code">
		                		<span class="description">The shortcode is populated instantly as you tweak any of the above fields. You don't need to hit any button.</span>
		                	</td>

		                </tr>
		            </tbody>
		        </table>
			</div>
		<?php
		return ob_get_clean();
	}

	/**
	 * Retrieve registered post types
	 *
	 * @return array Post types
	 */
	private function ess_get_post_types()
	{
		$post_types = array();
		$all_post_types = get_post_types( array( 'public' => true ), 'objects' );

		foreach ( $all_post_types as $slug => $post_type ) {
			$post_types[$slug] = $post_type->labels->name;
		}

		return $post_types;
	}
}